package com.contentsquare.flutter

import android.content.Context
import android.view.View
import com.contentsquare.android.Contentsquare
import com.contentsquare.android.api.bridge.flutter.FlutterInterface
import com.contentsquare.android.api.bridge.flutter.ExternalViewGraphListener
import com.contentsquare.android.api.model.Transaction
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel.Result


class MethodCallHandlerImpl {
    fun send(call: MethodCall, result: Result) {
        val screenName = call.argument<String>("screenName")
        if (screenName != null) {
            Contentsquare.send(screenName)
            result.success(0)
            return
        }
        result.error("MISSING_ARGUMENT", "Missing string for the screenName parameter", null)
    }

    fun sendTransaction(call: MethodCall, result: Result) {
        val price = call.argument<Double>("price")
        val currency = call.argument<String>("currency")
        val transactionId = call.argument<String>("transactionId")

        if (price == null) {
            result.error("MISSING_ARGUMENT", "Missing double for the price parameter", null)
            return
        }
        if (currency == null) {
            result.error("MISSING_ARGUMENT", "Missing string for the currency parameter", null)
            return
        }
        if (transactionId == null) {
            Contentsquare.send(Transaction.builder(price.toFloat(), currency).build())
            result.success(0)
            return
        }
        Contentsquare.send(Transaction.builder(price.toFloat(), currency).id(transactionId).build())
        result.success(0)
    }

    fun sendDynamicVar(call: MethodCall, result: Result) {
        val key = call.argument<String>("key")
        val stringValue = call.argument<String>("stringValue")
        val intValue = call.argument<Int>("intValue")

        if (key == null) {
            result.error("MISSING_ARGUMENT", "Missing string for the key parameter", null)
            return
        }
        if (stringValue != null) {
            Contentsquare.send(key, stringValue)
            result.success(0)
            return
        }
        if (intValue != null) {
            Contentsquare.send(key, intValue.toLong())
            result.success(0)
            return
        }
        result.error("MISSING_ARGUMENT", "Missing stringValue or intValue parameter", null)
    }

    fun optIn(result: Result, context: Context?) {
        if (context == null) {
            result.error("NO_CONTEXT", "Cannot call optIn without context, please wait for FlutterEngine to start. This can happen if you call the optIn method from the global scope or from outside a build method.", null)
            return
        }
        Contentsquare.optIn(context)
        result.success(0)
    }


    fun optOut(result: Result, context: Context?) {
        if (context == null) {
            result.error("NO_CONTEXT", "Cannot call optOut without context, please wait for FlutterEngine to start. This can happen if you call the optOut method from the global scope or from outside a build method.", null)
            return
        }
        Contentsquare.optOut(context)
        result.success(0)
    }

    fun forgetMe(result: Result) {
        Contentsquare.forgetMe()
        result.success(0)
    }

    fun getUserId(result: Result) {
        val userId = Contentsquare.getUserId()
        result.success(userId)
    }

    fun stopTracking(result: Result) {
        Contentsquare.stopTracking()
        result.success(0)
    }

    fun resumeTracking(result: Result) {
        Contentsquare.resumeTracking()
        result.success(0)
    }

    fun sendTouchEvent(call: MethodCall, result: Result) {
        val body = call.argument<String>("body")

        if (body == null) {
            result.error("MISSING_ARGUMENT", "Missing string for the body parameter", null)
            return
        }

        CSFlutter.sendEvent(body)

        result.success(0)
    }

    object CSFlutter {

        const val FLUTTER_SEPARATOR = "|flutter|"

        fun registerExternalView(view: View, listener: ExternalViewGraphListener) {
            try {
                val flutterInterface = FlutterInterface::class.java
                flutterInterface.getDeclaredMethod(
                    "registerExternalView", View::class.java, ExternalViewGraphListener::class.java
                ).run {
                    isAccessible = true
                    invoke(null, view, listener)
                }
            } catch (ex: Exception) {
                println(ex)
            }
        }

        fun unRegisterExternalView(view: View) {
            try {
                val flutterInterface = FlutterInterface::class.java
                flutterInterface.getDeclaredMethod(
                    "unRegisterExternalView", View::class.java
                ).run {
                    isAccessible = true
                    invoke(null, view)
                }
            } catch (ex: Exception) {
                println(ex)
            }
        }

        fun sendEvent(json: String) {
            try {
                val flutterInterface = FlutterInterface::class.java
                flutterInterface.getDeclaredMethod(
                        "sendEvent",
                        String::class.java
                ).run {
                    isAccessible = true
                    invoke(null, json)
                }
            } catch (ex: Exception) {
                println(ex)
            }
        }
    }
}
