import 'package:contentsquare/contentsquare.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '../../../../widgets/app_bar.dart';

class PrivacyScreen extends StatefulWidget {
  const PrivacyScreen({Key? key}) : super(key: key);

  @override
  State<PrivacyScreen> createState() => _PrivacyScreenState();
}

class _PrivacyScreenState extends State<PrivacyScreen> {
  String? userId;

  Future<T?> _guarded<T>(Future<T> Function() callback) async {
    try {
      return callback();
    } on PlatformException catch (e) {
      if (e.message != null) {
        ScaffoldMessenger.of(context)
            .showSnackBar(SnackBar(content: Text(e.message!)));
      }
    }
    return null;
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBar(title: 'Privacy'),
      body: Padding(
        padding: const EdgeInsets.only(top: 30.0),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            OutlinedButton(
              onPressed: () => _guarded(() => Contentsquare().optIn()),
              child: Text('Opt in'),
            ),
            SizedBox(height: 20),
            OutlinedButton(
              onPressed: () => _guarded(() => Contentsquare().optOut()),
              child: Text('Opt out'),
            ),
            SizedBox(height: 20),
            OutlinedButton(
              onPressed: () => _guarded(() => Contentsquare().forgetMe()),
              child: Text('Forget me'),
            ),
            SizedBox(height: 20),
            OutlinedButton(
              onPressed: () => _guarded(() => Contentsquare().stopTracking()),
              child: Text('Stop tracking'),
            ),
            SizedBox(height: 20),
            OutlinedButton(
              onPressed: () => _guarded(() => Contentsquare().resumeTracking()),
              child: Text('Resume tracking'),
            ),
            SizedBox(height: 20),
            OutlinedButton(
              onPressed: () async {
                final _userId =
                    await _guarded(() => Contentsquare().getUserId());
                setState(() => userId = _userId ?? 'Could not get user id');
              },
              child: Text('Get user id'),
            ),
            SizedBox(height: 20),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 30.0),
              child: Divider(color: Colors.black, thickness: 1),
            ),
            SizedBox(height: 20),
            if (userId != null)
              Center(child: Text('User id: $userId')),
          ],
        ),
      ),
    );
  }
}
