import 'package:auto_route/auto_route.dart';
import 'package:contentsquare/contentsquare.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

// The RouteObserver to use as a navigation observer.
RouteObserver<ModalRoute<void>> getRouteObserver() =>
    RouteObserver<ModalRoute<void>>();

/// Wrap all your screens inside this widget to have them automatically set in
/// the contentsquare API when they are at the top of the navigation stack.
///
/// You can also call [ContentsquareScreenMonitor.sendScreenView] manually to
/// send the screen view.
class ContentsquareScreenMonitor extends StatefulWidget {
  const ContentsquareScreenMonitor({
    Key? key,
    required this.child,
    this.screenName,
  }) : super(key: key);

  final Widget child;

  /// The name of the screen to monitor.
  ///
  /// If not provided, the screen name will be the runtimeType of the child.
  final String? screenName;

  @override
  State<ContentsquareScreenMonitor> createState() =>
      ContentsquareScreenMonitorState();

  /// [screenName] will be used as the name sent to contentsquare.
  static Future<void> sendScreenView(
    BuildContext context, {
    required String screenName,
  }) async {
    print('sendScreenView: $screenName');

    try {
      await Contentsquare().send(screenName);
    } on PlatformException catch (e) {
      if (e.message != null) {
        ScaffoldMessenger.of(context)
            .showSnackBar(SnackBar(content: Text(e.message!)));
      }
    }
  }
}

// Implement RouteAware in a widget's state and subscribe it to the RouteObserver.
class ContentsquareScreenMonitorState extends State<ContentsquareScreenMonitor>
    with AutoRouteAware {
  String get _screenName =>
      widget.screenName ?? widget.child.runtimeType.toString();

  AutoRouteObserver? _observer;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _observer =
        RouterScope.of(context).firstObserverOfType<AutoRouteObserver>();
    if (_observer != null) {
      _observer!.subscribe(this, context.routeData);
    }
  }

  @override
  void dispose() {
    super.dispose();
    _observer?.unsubscribe(this);
  }

  @override
  void didPush() {
    ContentsquareScreenMonitor.sendScreenView(context, screenName: _screenName);
  }

  @override
  void didPopNext() {
    ContentsquareScreenMonitor.sendScreenView(context, screenName: _screenName);
  }

  @override
  Widget build(BuildContext context) => widget.child;
}
