import 'dart:convert';
import 'dart:io';

import 'package:contentsquare/src/modules/screengraph/compression.dart';

/// A class which decompresses the TVP
///
/// This is the invert function of [compressTVP].
///
/// [tvp] must start with [nativePath]
String decompressTVP({
  required String tvp,
  String nativePath = '',
}) {
  assert(tvp.startsWith(nativePath));

  final flutterTVP = tvp.replaceRange(0, nativePath.length, '');

  final codec = utf8.fuse(zlib.fuse(base64));
  final decompressedFlutterTVP = codec.decode(flutterTVP);

  return nativePath + decompressedFlutterTVP;
}

List<Map<String, Object?>> decompressScreengraph({
  required String nativePath,
  required String screengraphString,
}) {
  final screengraphMapList =
      (jsonDecode(screengraphString) as List).cast<Map<String, dynamic>>();

  final decompressedScreengraphMap = screengraphMapList
      .map(
        (screengraphMap) => _decompressScreengraphMap(
          nativePath: nativePath,
          screengraphMap: screengraphMap,
        ),
      )
      .toList();

  return decompressedScreengraphMap;
}

Map<String, dynamic> _decompressScreengraphMap({
  required String nativePath,
  required Map<String, dynamic> screengraphMap,
}) {
  final screengraphMapCopy = Map<String, dynamic>.from(screengraphMap);

  final compressedChildren =
      (screengraphMapCopy['children'] as List).cast<Map<String, dynamic>>();
  final decompressedChildren = compressedChildren
      .map(
        (compressedChild) => _decompressScreengraphMap(
          nativePath: nativePath,
          screengraphMap: compressedChild,
        ),
      )
      .toList();

  screengraphMapCopy['metadata']['fullpath'] = decompressTVP(
    nativePath: nativePath,
    tvp: screengraphMap['metadata']['fullpath'],
  );
  screengraphMapCopy['children'] = decompressedChildren;

  return screengraphMapCopy;
}
