part of 'matcher.dart';

/// A json encoder which:
///   - Use indents and newlines to make the json more readable when working with
///     maps.
///   - Use list of string for multi-line strings (because multi-line strings are
///     not supported in json).
class _FixtureEncoder extends Converter<Object?, Object?> {
  const _FixtureEncoder();

  @override
  Object? convert(Object? fixture) {
    return _convertMultiLineStringToList(fixture);
  }

  /// Converts values of a maps so that multi-line strings are converted to
  /// lists.
  ///
  ///
  /// This function is applied recursively if a value of the map is a map.
  ///
  ///
  /// Example:
  /// ```dart
  /// // Input:
  /// { "key": "my\nvalue" }
  ///
  /// // Output:
  /// { "key": ["my", "value"] }
  /// ```
  Object? _convertMultiLineStringToList(Object? value) {
    if (value is Map) {
      return value.map(
        (key, value) => MapEntry(key, _convertMultiLineStringToList(value)),
      );
    }
    if (value is String) {
      final isMultiLine = value.contains('\n');
      return isMultiLine ? value.split('\n') : value;
    }
    return value;
  }
}
