import 'package:contentsquare/src/modules/screengraph/element_extension.dart';
import 'package:contentsquare/src/modules/screengraph/element_tree.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_test/flutter_test.dart';

import '../../../helpers/matchers/fixtures/matcher.dart';

main() {
  testWidgets(
    'The root element of the tree should always be a [RenderObjectToWidgetElement]',
    (WidgetTester tester) async {
      await tester.pumpWidget(const MaterialApp(home: Text('Hello')));

      final rootElement = getRootElement();

      expect(rootElement, isA<RenderObjectToWidgetElement>());
    },
  );

  testWidgets(
    'The root element of the tree is associated with a render object of type [RenderView]',
    (WidgetTester tester) async {
      await tester.pumpWidget(const MaterialApp(home: Text('Hello')));

      final rootElement = getRootElement();

      expect(rootElement.directRenderObject, isA<RenderView>());
    },
  );

  testWidgets(
    'The string representation of the root element of the tree should match the fixture',
    (WidgetTester tester) async {
      await tester.pumpWidget(const MaterialApp(home: Text('Hello')));

      final rootElement = getRootElementTree();

      final rootElementTreeString = rootElement.toString();

      expect(
        rootElementTreeString,
        matchesFixture('elementTreeString'),
      );
    },
  );
}
